package org.greenstone.gsdl3.util;

import java.util.Date;
import java.util.Locale;
import java.text.SimpleDateFormat;

/** a class to contain various static methods that are used by the xslt
 * stylesheets
 */
public class XSLTUtil {

    public static boolean equals(String s1, String s2) {
	return s1.equals(s2);
    }

    public static boolean exists(String s1, String s2) {
	return !s1.equals("");
    }
    public static boolean contains(String s1, String s2) {
	return (s1.indexOf(s2) != -1);
    }
    public static boolean startsWith(String s1, String s2) {
	return s1.startsWith(s2);
    }
    public static boolean endsWith(String s1, String s2) {
	return s1.endsWith(s2);
    }
    
    public static String toLower(String orig) {
	return orig.toLowerCase();
    }
    public static String toUpper(String orig) {
	return orig.toUpperCase();
    }

    public static byte[] toUTF8(String orig) {
	
	try {
	    byte[] utf8 = orig.getBytes("UTF-8");
	    return utf8;
	}
	catch (Exception e){
	    System.err.println("XSLTUtil.toUTF8: unsupported encoding");
	    return orig.getBytes();
	}
    }

    public static String getNumberedItem(String list, int number) {
	String [] items = list.split(",", -1);	
	if (items.length > number) {
	    return items[number];
	}
	return ""; // index out of bounds
    }

    
    public static String tidyWhitespace(String original) {
	
	if (original==null || original.equals("")) {
	    return original;
	}
	String new_s = original.replaceAll("\\s+", " ");
	return new_s;
    }


    public static String getInterfaceText(String interface_name, String lang, String key) {
	return getInterfaceText(interface_name, lang, key, null);
    }
    public static String getInterfaceText(String interface_name, String lang, String key, String args_str) {
	String [] args = null;
	if (args_str!=null && !args_str.equals("")) {
	    args = args_str.split(";");
	}
	Dictionary dict = new Dictionary("interface_"+interface_name, lang);
        String result = dict.get(key, args);

	if (result == null && !interface_name.equals("default")) { // not found, try the default interface
	    dict = new Dictionary("interface_default", lang);
	    result = dict.get(key, args);
	}
	    
        if (result == null) { // not found
            return "_"+key+"_";
        }
        return result;
    }

    public static boolean isImage(String mimetype) {
	if (mimetype.startsWith("image/")) {
	    return true;
	}
	return false;
    }

    public static String formatDate(String date, String lang) {
	
	String in_pattern = "yyyyMMdd";
	String out_pattern = "dd MMMM yyyy";
	if (date.length()==6) {
	    in_pattern = "yyyyMM";
	}
	
	SimpleDateFormat formatter = new SimpleDateFormat(in_pattern, new Locale(lang));
	try {
	    Date d = formatter.parse(date);
	    formatter.applyPattern(out_pattern);
	    String new_date = formatter.format(d);
	    return new_date;
	} catch (Exception e) {
	    return date;
	}
	
    }

    public static String formatLanguage(String display_lang, String lang) {

	return new Locale(display_lang).getDisplayLanguage(new Locale(lang));
    }

    public static String cgiSafe(String original, String lang) {

	original = original.replace('&', ' ');
	original = original.replaceAll(" ", "%20");
	return original;
    }
}

